package org.openxma.dsl.generator.helper;

import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.generator.GeneratorConfiguration;
import org.openxma.dsl.pom.PomFactory;
import org.openxma.dsl.pom.model.ComposeData;
import org.openxma.dsl.pom.model.Composite;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.model.TabulatorPosition;
import org.openxma.dsl.pom.scoping.PomElementCollector;
import org.openxma.dsl.pom.util.StyleAccess;


public class TabulatorExtension {		
	
	public static void initTabulator(Composite composite, String style) {
		
		ComposeData composeData = FormDataExtension.getComposeData(composite);
		
		if (FormDataExtension.hasComposeData(composite)==false) return;
		
		// Use explicitly defined tabulators if available
		if (composeData != null && composeData.getTabulators().size()>0 && isTabulatorFromLayoutStrategy(composite)==false ) {		
			return;
		}
		
		Composite parentComposite = PomElementCollector.getContainerWithComposeData(composite);
		if (parentComposite != null) {
			ComposeData pcd = FormDataExtension.getComposeData(parentComposite);
			if (pcd != null && pcd.getTabulators().size() > 0 && isTabulatorFromLayoutStrategy(parentComposite) == false) {
				if (composeData == null) {
				    composeData = createComposeDataAndSetStylesFromComposite(composite); 
				}
				composeData.getTabulators().addAll(EcoreUtil.copyAll(pcd.getTabulators()));
				return;
			}
		}
		
		List<TabulatorPosition> definedTabulatorPositions = null;
		if (composeData != null) {
			definedTabulatorPositions = composeData.getTabulators();
		}
		
		List<TabulatorPosition> tabs = GeneratorConfiguration.INSTANCE.getLayoutStrategy().getTabulatorPositions(composite, definedTabulatorPositions, style);	
		if (tabs != null && tabs.size()>0) {			
			if (composeData == null) {
				composeData = createComposeDataAndSetStylesFromComposite(composite); 
			} 		
			setTabulatorIsFromLayoutStrategy(composite);
			composeData.getTabulators().addAll(tabs);
		}
	}
	
	private static ComposeData createComposeDataAndSetStylesFromComposite(Composite composite) {
        ComposeData composeData = PomFactory.eINSTANCE.createComposeData();
        Map<Integer,StyleSpecificationProperty> stylePropertiesOfComposite = StyleAccess.getCombinedStyleProperty(composite);
        if (stylePropertiesOfComposite != null) { 
            StyleAccess.setCombinedStyleProperty(composeData, stylePropertiesOfComposite);
        }
        FormDataExtension.setComposeData(composite,composeData);	
        return composeData;	    
	}
	
	private static boolean isTabulatorFromLayoutStrategy(EObject obj) {
		Object value = EObjectPropertiesAdapter.getProperty(obj,"tabulatorIsFromLayoutStrategy");
		if (value instanceof Boolean) {
			return ((Boolean)value).booleanValue();
		}
		return false;
	}
	
	private static void setTabulatorIsFromLayoutStrategy(EObject obj) {
		if (obj != null) {
			EObjectPropertiesAdapter.setProperty(obj,"tabulatorIsFromLayoutStrategy",new Boolean(true));
		}
	}
}
